/**
 * compute a global alignment using the Needleman Wunsch algorithm
 * @author Widmann Annekathrin und Riester Markus
 * Date: Oct 20, 2003
 */
package albi.align;

import java.util.*;

/** 
 * compute a global alignment using the Needleman Wunsch algorithm
 */
public class NeedlemanWunsch extends NeedlemanWunschBase {
    /**
     * compute a global alignment using the Needleman Wunsch algorithm
     */
    private int[][] M;
	Stack aligned1 = new Stack();
	Stack aligned2 = new Stack();
	
    public void compute ()
    {
        // 1. base case:
		M = new int[getSequence1().length()+1][getSequence2().length()+1];
				
		//set borders		
		for (int i = 0; i < getSequence1().length()+1; i++) M[i][0] = i * (-1) * getGapPenalty();
		for (int j = 0; j < getSequence2().length()+1; j++) M[0][j] = j * (-1) * getGapPenalty();
		
		//fill matrix
		for (int i = 1; i < getSequence1().length()+1; i++)
			for (int j = 1; j < getSequence2().length()+1; j++) {
			
				M[i][j] = max(
									 M[i-1][j] - getGapPenalty(),
									 M[i][j-1] - getGapPenalty(),
									 M[i-1][j-1] + match(getSequence1().charAt(i-1),getSequence2().charAt(j-1))	
									);
   			}		
		
		// 2. recursion:				
		align(getSequence1().length(),getSequence2().length());
	
		//	4. pop characters from stacks to get strings
		String tmp_aligned1 = ""; String tmp_aligned2 = "";
		while (true) {
			try {
				tmp_aligned1 += aligned1.pop();
			} catch (EmptyStackException e) {
				break;
			}
		}		
	
		while (true) {
			try {
				tmp_aligned2 += aligned2.pop();
			} catch (EmptyStackException e) {
				break;
			}
		}		
		
		// 5. set result using setAligned1, setAligned2 and setScore
		setAligned1(tmp_aligned1);
		setAligned2(tmp_aligned2);
		setScore(M[getSequence1().length()][getSequence2().length()]);							
    }
    
   	 private void align(int i, int j) {
		//	3. trace back, push characters onto stack
   	 	 if (i >= 0 || j >= 0) { // not yet in upper left corner (0,0)? then ... 	 	
   	 		if (i > 0 && j > 0 && M[i][j] == M[i-1][j-1] + match(getSequence1().charAt(i-1),getSequence2().charAt(j-1))) {
				aligned1.push(String.valueOf(getSequence1().charAt(i-1)));
				aligned2.push(String.valueOf(getSequence2().charAt(j-1)));
				align(i-1,j-1);			
			} else if (j > 0 && M[i][j] == M[i][j-1] - getGapPenalty()) {
				aligned1.push("-");
				aligned2.push(String.valueOf(getSequence2().charAt(j-1)));
				align(i,j-1);								
			} else  if (i > 0) {
				aligned1.push(String.valueOf(getSequence1().charAt(i-1)));
				aligned2.push("-");
				align(i-1,j);
			}
   	 	}		
   	 }
}
