package albi.io;
/**
  * Base class for FastA i/o
 * Daniel Huson, 12.10.2003
 */


import java.io.*;
import java.util.Vector;

/**
 * FastA i/o base class
 */
public class FastABase {
    private int size=0;
    private Vector headers=new Vector();
    private Vector sequences=new Vector();



    /**
     * add a header and sequence
     * @param header
     * @param sequence
     */
    public void add (String header, String sequence) {
        set(getSize(),header,sequence);
    }

    /**
     * clear the data
     */
    public void clear() {
        headers.clear();
        sequences.clear();
        size=0;
    }

    /**
     * gets the header
     * @param i the index
     * @return
     */
    public String getHeader(int i) {
        return (String)headers.get(i);
    }

    /**
     * sets the header and sequence
     * @param i the index
     * @param header
     */
    public void set (int i,String header,String sequence) {
        if(i<getSize())
        {
            this.headers.set(i,header);
            this.sequences.set(i,sequence);
        }
        else
        {
            setSize(i+1);
            this.headers.add(i,header);
            this.sequences.add(i,sequence);
        }
    }

    /**
     * gets the sequence
     * @param i the index
     * @return           the sequence
     */
    public String getSequence(int i) {
        return (String)sequences.get(i);
    }

    /**
     * gets the first header
     * @return first header
     */
    public String getFirstHeader ()
    {
        return (String)headers.firstElement();
    }

    /**
     * gets the first sequence
     * @return first sequence
     */
    public String getFirstSequence ()
    {
        return (String)sequences.firstElement();
    }

    /**
     * sets the size of sequences
     * @param n the size
     */
    public void setSize (int n)
    {
        if(n>size)
        {
            headers.setSize(n);
            sequences.setSize(n);
        }
        size=n;
    }

    /**
     * get the size of sequences
     * @return size of sequences
     */
    public int getSize ()
    {
        return size;
    }
}
