/**
 * report a global alignment using the Needleman Wunsch algorithm
 * @author Daniel Huson
 * Date: Oct 20, 2003
 */
package albi.progs;

import albi.io.FastA;
import albi.align.NeedlemanWunsch;

import java.io.File;
import java.io.FileReader;


/** 
 * report a global alignment computed using the Needleman Wunsch algorithm
 */
public class RunNeedlemanWunsch  {
    /**
     * runs the Needleman Wunsch algorithm.
     * Commandline arguments: file1 file matchscore mismatchscore gappenalty
     * @param args command line arguments
     * @throws Exception
     */
    static public void main (String[] args) throws Exception
    {
        if(args.length!=5)
            throw new Exception("Usage: file1 file2 matchscore mismatchscore gappenalty");
        FastA fasta1=new FastA();
        fasta1.read(new FileReader(new File(args[0])));
        FastA fasta2=new FastA();
        fasta2.read(new FileReader(new File(args[1])));

        // setup algorithm parameters:
        NeedlemanWunsch nw=new NeedlemanWunsch();
        nw.setSequence1(fasta1.getFirstSequence());
        nw.setSequence2(fasta2.getFirstSequence());
        nw.setMatchScore(Integer.parseInt(args[2]));
        nw.setMismatchScore(Integer.parseInt(args[3]));
        nw.setGapPenalty(Integer.parseInt(args[4]));

        // Display input:
        System.out.println("Input:");
        System.out.println("seq1="+nw.getSequence1());
        System.out.println("seq2="+nw.getSequence2());

        // run algorithm:
        nw.compute();

        // Display alignment:
        System.out.println("Alignment:");
        System.out.println("seq1="+nw.getAligned1());
        System.out.println("seq2="+nw.getAligned2());

        System.out.println("Score="+nw.getScore());
    }
}
