/**
 * Read/write sequences in fastA format
 * @author huson
 * Date: Oct 12, 2003
 */
package albi.io;

import java.io.*;


/** 
 * Read/write sequences in fastA format
 */
public class FastA extends FastABase {
        /**
     * read header and sequence in fastA format
     * @param r
     * @throws java.io.IOException
     */
    public void read (Reader r) throws IOException
    {
        clear();

        BufferedReader br=new BufferedReader(r);

        String header="";
        StringBuffer sequence=null;

        String aLine=br.readLine();

        while(aLine!=null)
        {
            aLine=aLine.trim();
            if(aLine.length()>0)
            {

                if(aLine.charAt(0)=='>') // new fasta header
                {
                    if(header.length()>0)
                    {
                        add(header,sequence.toString());
                    }
                    header=aLine.substring(1);
                    if(header!=null)
                        header=header.trim();
                    sequence=new StringBuffer();
                }
                else
                    sequence.append(aLine);
            }
            aLine=br.readLine();
        }
        if(header.length()>0)
        {
            add(header,sequence.toString());
        }
    }

    /**
     * write header and sequence in fastA format
     * @param w
     * @throws IOException
     */
    public void write (Writer w) throws IOException
    {
        for(int i=0;i<getSize();i++)
        {
            if(getHeader(i)!=null)
            {
                w.write("> "+getHeader(i)+"\n");
                for(int c=0;c<getSequence(i).length();c+=60)
                {
                    int d=Math.min(getSequence(i).length(),c+60);
                    w.write(getSequence(i).substring(c,d)+"\n");
                }
            }
        }
    }

    /**
     * reads sequences from a file and write them to standard output
     */
    public static void main (String args[]) throws Exception
    {
        if(args.length!=1)
            throw new Exception("Usage: FastA infile");

        FastA fasta=new FastA();

        fasta.read(new FileReader(new File(args[0])));

        StringWriter sr=new StringWriter();

        fasta.write(sr);
        System.out.print(sr);
    }
}
